@@ -3,8 +3,6 @@ import argparse
 import functools
 import json
 import os
-import re
-import stat
 import sys
 from io import TextIOWrapper
 from pathlib import Path
@@ -15,14 +13,42 @@ from . import __version__, api, sections
 from .exceptions import FileSkipped
 from .logo import ASCII_ART
 from .profiles import profiles
-from .settings import SUPPORTED_EXTENSIONS, VALID_PY_TARGETS, Config, WrapModes
+from .settings import VALID_PY_TARGETS, Config, WrapModes
 
 try:
     from .setuptools_commands import ISortCommand  # noqa: F401
 except ImportError:
     pass
 
-shebang_re = re.compile(br"^#!.*\bpython[23w]?\b")
+DEPRECATED_SINGLE_DASH_ARGS = {
+    "-ac",
+    "-af",
+    "-ca",
+    "-cs",
+    "-df",
+    "-ds",
+    "-dt",
+    "-fas",
+    "-fass",
+    "-ff",
+    "-fgw",
+    "-fss",
+    "-lai",
+    "-lbt",
+    "-le",
+    "-ls",
+    "-nis",
+    "-nlb",
+    "-ot",
+    "-rr",
+    "-sd",
+    "-sg",
+    "-sl",
+    "-sp",
+    "-tc",
+    "-wl",
+    "-ws",
+}
 QUICK_GUIDE = f"""
 {ASCII_ART}
 
@@ -39,32 +65,6 @@ Visit https://timothycrosley.github.io/isort/ for complete information about how
 """
 
 
-def is_python_file(path: str) -> bool:
-    _root, ext = os.path.splitext(path)
-    if ext in SUPPORTED_EXTENSIONS:
-        return True
-    if ext in (".pex",):
-        return False
-
-    # Skip editor backup files.
-    if path.endswith("~"):
-        return False
-
-    try:
-        if stat.S_ISFIFO(os.stat(path).st_mode):
-            return False
-    except OSError:
-        pass
-
-    try:
-        with open(path, "rb") as fp:
-            line = fp.readline(100)
-    except OSError:
-        return False
-    else:
-        return bool(shebang_re.match(line))
-
-
 class SortAttempt:
     def __init__(self, incorrectly_sorted: bool, skipped: bool) -> None:
         self.incorrectly_sorted = incorrectly_sorted
@@ -121,14 +121,15 @@ def iter_source_code(paths: Iterable[str], config: Config, skipped: List[str]) -
 
                     resolved_path = full_path.resolve()
                     if resolved_path in visited_dirs:  # pragma: no cover
-                        warn(f"Likely recursive symlink detected to {resolved_path}")
+                        if not config.quiet:
+                            warn(f"Likely recursive symlink detected to {resolved_path}")
                         dirnames.remove(dirname)
                     else:
                         visited_dirs.add(resolved_path)
 
                 for filename in filenames:
                     filepath = os.path.join(dirpath, filename)
-                    if is_python_file(filepath):
+                    if config.is_supported_filetype(filepath):
                         if config.is_skipped(Path(filepath)):
                             skipped.append(filename)
                         else:
@@ -326,6 +327,13 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         dest="length_sort",
         action="store_true",
     )
+    parser.add_argument(
+        "--lss",
+        "--length-sort-straight",
+        help="Sort straight imports by their string length.",
+        dest="length_sort_straight",
+        action="store_true",
+    )
     parser.add_argument(
         "-m",
         "--multi-line",
@@ -334,7 +342,9 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         + [str(mode.value) for mode in WrapModes.__members__.values()],
         type=str,
         help="Multi line output (0-grid, 1-vertical, 2-hanging, 3-vert-hanging, 4-vert-grid, "
-        "5-vert-grid-grouped, 6-vert-grid-grouped-no-comma).",
+        "5-vert-grid-grouped, 6-vert-grid-grouped-no-comma, 7-noqa, "
+        "8-vertical-hanging-indent-bracket, 9-vertical-prefix-from-module-import, "
+        "10-hanging-indent-with-parentheses).",
     )
     parser.add_argument(
         "-n",
@@ -452,7 +462,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--skip-gitignore",
         action="store_true",
         dest="skip_gitignore",
-        help="Treat project as a git respository and ignore files listed in .gitignore",
+        help="Treat project as a git repository and ignore files listed in .gitignore",
     )
     inline_args_group.add_argument(
         "--sl",
@@ -635,11 +645,23 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--float-to-top",
         dest="float_to_top",
         action="store_true",
-        help="Causes all non indented imports to float to the top of the file having its imports "
-        "sorted. *NOTE*: This is an **experimental** feature. It currently doesn't work with "
-        "cimports and is guaranteed to run much slower and use much more memory than the default. "
-        "Still it can be a great shortcut for collecting imports every once in a while when you put"
-        " them in the middle of a file.",
+        help="Causes all non-indented imports to float to the top of the file having its imports "
+        "sorted.  It can be an excellent shortcut for collecting imports every once in a while "
+        "when you place them in the middle of a file to avoid context switching.\n\n"
+        "*NOTE*: It currently doesn't work with cimports and introduces some extra over-head "
+        "and a performance penalty.",
+    )
+    parser.add_argument(
+        "--treat-comment-as-code",
+        dest="treat_comments_as_code",
+        action="append",
+        help="Tells isort to treat the specified single line comment(s) as if they are code.",
+    )
+    parser.add_argument(
+        "--treat-all-comment-as-code",
+        dest="treat_all_comments_as_code",
+        action="store_true",
+        help="Tells isort to treat all single line comments as if they are code.",
     )
     parser.add_argument(
         "--formatter",
@@ -647,6 +669,27 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         type=str,
         help="Specifies the name of a formatting plugin to use when producing output.",
     )
+    parser.add_argument(
+        "--ext",
+        "--extension",
+        "--supported-extension",
+        dest="supported_extensions",
+        action="append",
+        help="Specifies what extensions isort can be ran against.",
+    )
+    parser.add_argument(
+        "--blocked-extension",
+        dest="blocked_extensions",
+        action="append",
+        help="Specifies what extensions isort can never be ran against.",
+    )
+    parser.add_argument(
+        "--dedup-headings",
+        dest="dedup_headings",
+        action="store_true",
+        help="Tells isort to only show an identical custom import heading comment once, even if"
+        " there are multiple sections with the comment set.",
+    )
 
     # deprecated options
     parser.add_argument(
@@ -684,14 +727,25 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         const="--keep-direct-and-as",
         help=argparse.SUPPRESS,
     )
+
     return parser
 
 
 def parse_args(argv: Optional[Sequence[str]] = None) -> Dict[str, Any]:
+    argv = sys.argv[1:] if argv is None else list(argv)
+    remapped_deprecated_args = []
+    for index, arg in enumerate(argv):
+        if arg in DEPRECATED_SINGLE_DASH_ARGS:
+            remapped_deprecated_args.append(arg)
+            argv[index] = f"-{arg}"
+
     parser = _build_arg_parser()
     arguments = {key: value for key, value in vars(parser.parse_args(argv)).items() if value}
+    if remapped_deprecated_args:
+        arguments["remapped_deprecated_args"] = remapped_deprecated_args
     if "dont_order_by_type" in arguments:
         arguments["order_by_type"] = False
+        del arguments["dont_order_by_type"]
     multi_line_output = arguments.get("multi_line_output", None)
     if multi_line_output:
         if multi_line_output.isdigit():
@@ -740,18 +794,9 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
     if not file_names and not show_config:
         print(QUICK_GUIDE)
         if arguments:
-            sys.exit(f"Error: arguments passed in without any paths or content.")
+            sys.exit("Error: arguments passed in without any paths or content.")
         else:
             return
-    elif file_names == ["-"] and not show_config:
-        arguments.setdefault("settings_path", os.getcwd())
-        api.sort_stream(
-            input_stream=sys.stdin if stdin is None else stdin,
-            output_stream=sys.stdout,
-            **arguments,
-        )
-        return
-
     if "settings_path" not in arguments:
         arguments["settings_path"] = (
             os.path.abspath(file_names[0] if file_names else ".") or os.getcwd()
@@ -766,13 +811,8 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
     show_diff = config_dict.pop("show_diff", False)
     write_to_stdout = config_dict.pop("write_to_stdout", False)
     deprecated_flags = config_dict.pop("deprecated_flags", False)
-
-    if deprecated_flags:  # pragma: no cover
-        warn(
-            f"\n\nThe following deprecated CLI flags were used: {', '.join(deprecated_flags)}!\n"
-            "Please see the 5.0.0 upgrade guide:\n"
-            "\thttps://timothycrosley.github.io/isort/docs/upgrade_guides/5.0.0/\n"
-        )
+    remapped_deprecated_args = config_dict.pop("remapped_deprecated_args", False)
+    wrong_sorted_files = False
 
     if "src_paths" in config_dict:
         config_dict["src_paths"] = {
@@ -783,74 +823,98 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
     if show_config:
         print(json.dumps(config.__dict__, indent=4, separators=(",", ": "), default=_preconvert))
         return
-
-    wrong_sorted_files = False
-    skipped: List[str] = []
-
-    if config.filter_files:
-        filtered_files = []
-        for file_name in file_names:
-            if config.is_skipped(Path(file_name)):
-                skipped.append(file_name)
-            else:
-                filtered_files.append(file_name)
-        file_names = filtered_files
-
-    file_names = iter_source_code(file_names, config, skipped)
-    num_skipped = 0
-    if config.verbose:
-        print(ASCII_ART)
-
-    if jobs:
-        import multiprocessing
-
-        executor = multiprocessing.Pool(jobs)
-        attempt_iterator = executor.imap(
-            functools.partial(
-                sort_imports,
-                config=config,
-                check=check,
-                ask_to_apply=ask_to_apply,
-                write_to_stdout=write_to_stdout,
-            ),
-            file_names,
+    elif file_names == ["-"]:
+        arguments.setdefault("settings_path", os.getcwd())
+        api.sort_stream(
+            input_stream=sys.stdin if stdin is None else stdin,
+            output_stream=sys.stdout,
+            **arguments,
         )
     else:
-        # https://github.com/python/typeshed/pull/2814
-        attempt_iterator = (
-            sort_imports(  # type: ignore
-                file_name,
-                config=config,
-                check=check,
-                ask_to_apply=ask_to_apply,
-                show_diff=show_diff,
-                write_to_stdout=write_to_stdout,
-            )
-            for file_name in file_names
-        )
+        skipped: List[str] = []
 
-    for sort_attempt in attempt_iterator:
-        if not sort_attempt:
-            continue  # pragma: no cover - shouldn't happen, satisfies type constraint
-        incorrectly_sorted = sort_attempt.incorrectly_sorted
-        if arguments.get("check", False) and incorrectly_sorted:
-            wrong_sorted_files = True
-        if sort_attempt.skipped:
-            num_skipped += 1  # pragma: no cover - shouldn't happen, due to skip in iter_source_code
+        if config.filter_files:
+            filtered_files = []
+            for file_name in file_names:
+                if config.is_skipped(Path(file_name)):
+                    skipped.append(file_name)
+                else:
+                    filtered_files.append(file_name)
+            file_names = filtered_files
+
+        file_names = iter_source_code(file_names, config, skipped)
+        num_skipped = 0
+        if config.verbose:
+            print(ASCII_ART)
+
+        if jobs:
+            import multiprocessing
+
+            executor = multiprocessing.Pool(jobs)
+            attempt_iterator = executor.imap(
+                functools.partial(
+                    sort_imports,
+                    config=config,
+                    check=check,
+                    ask_to_apply=ask_to_apply,
+                    write_to_stdout=write_to_stdout,
+                ),
+                file_names,
+            )
+        else:
+            # https://github.com/python/typeshed/pull/2814
+            attempt_iterator = (
+                sort_imports(  # type: ignore
+                    file_name,
+                    config=config,
+                    check=check,
+                    ask_to_apply=ask_to_apply,
+                    show_diff=show_diff,
+                    write_to_stdout=write_to_stdout,
+                )
+                for file_name in file_names
+            )
+
+        for sort_attempt in attempt_iterator:
+            if not sort_attempt:
+                continue  # pragma: no cover - shouldn't happen, satisfies type constraint
+            incorrectly_sorted = sort_attempt.incorrectly_sorted
+            if arguments.get("check", False) and incorrectly_sorted:
+                wrong_sorted_files = True
+            if sort_attempt.skipped:
+                num_skipped += (
+                    1  # pragma: no cover - shouldn't happen, due to skip in iter_source_code
+                )
+
+        num_skipped += len(skipped)
+        if num_skipped and not arguments.get("quiet", False):
+            if config.verbose:
+                for was_skipped in skipped:
+                    warn(
+                        f"{was_skipped} was skipped as it's listed in 'skip' setting"
+                        " or matches a glob in 'skip_glob' setting"
+                    )
+            print(f"Skipped {num_skipped} files")
+
+    if not config.quiet and (remapped_deprecated_args or deprecated_flags):
+        if remapped_deprecated_args:
+            warn(
+                "W0502: The following deprecated single dash CLI flags were used and translated: "
+                f"{', '.join(remapped_deprecated_args)}!"
+            )
+        if deprecated_flags:
+            warn(
+                "W0501: The following deprecated CLI flags were used and ignored: "
+                f"{', '.join(deprecated_flags)}!"
+            )
+        warn(
+            "W0500: Please see the 5.0.0 Upgrade guide: "
+            "https://timothycrosley.github.io/isort/docs/upgrade_guides/5.0.0/"
+        )
 
     if wrong_sorted_files:
         sys.exit(1)
 
-    num_skipped += len(skipped)
-    if num_skipped and not arguments.get("quiet", False):
-        if config.verbose:
-            for was_skipped in skipped:
-                warn(
-                    f"{was_skipped} was skipped as it's listed in 'skip' setting"
-                    " or matches a glob in 'skip_glob' setting"
-                )
-        print(f"Skipped {num_skipped} files")
-
 
 if __name__ == "__main__":
     main()
